@@ -12,7 +12,7 @@ module FakeFS
       @path = string
       @open = true
       @pointer = 0
-      @contents = [ '.', '..', ] + FileSystem.find(@path).values
+      @contents = [ '.', '..', ] + FileSystem.find(@path).entries
     end
 
     def close
@@ -57,8 +57,8 @@ module FakeFS
       @contents[integer]
     end
 
-    def self.[](pattern)
-      glob(pattern)
+    def self.[](*pattern)
+      glob pattern
     end
 
     def self.exists?(path)
@@ -75,7 +75,7 @@ module FakeFS
 
     def self.delete(string)
       _check_for_valid_file(string)
-      raise Errno::ENOTEMPTY, "Directory not empty - #{string}" unless FileSystem.find(string).values.empty?
+      raise Errno::ENOTEMPTY, "Directory not empty - #{string}" unless FileSystem.find(string).empty?
 
       FileSystem.delete(string)
     end
@@ -91,20 +91,27 @@ module FakeFS
     end
 
     def self.glob(pattern, &block)
-      files = [FileSystem.find(pattern) || []].flatten.map(&:to_s).sort
-      block_given? ? files.each { |file| block.call(file) } : files
+      matches_for_pattern = lambda do |matcher|
+        [FileSystem.find(matcher) || []].flatten.map{|e|
+          Dir.pwd.match(%r[\A/?\z]) || !e.to_s.match(%r[\A#{Dir.pwd}/?]) ? e.to_s : e.to_s.match(%r[\A#{Dir.pwd}/?]).post_match}.sort
+      end
+
+      if pattern.is_a? Array
+        files = pattern.collect { |matcher| matches_for_pattern.call matcher }.flatten
+      else
+        files = matches_for_pattern.call pattern
+      end
+      return block_given? ? files.each { |file| block.call(file) } : files
+    end
+
+    if RUBY_VERSION >= "1.9"
+      def self.home(user = nil)
+        RealDir.home(user)
+      end
     end
 
     def self.mkdir(string, integer = 0)
-      parent = string.split('/')
-      parent.pop
-
-      joined_parent_path = parent.join
-
-      _check_for_valid_file(joined_parent_path) unless joined_parent_path == ""
-      raise Errno::EEXIST, "File exists - #{string}" if File.exists?(string)
-
-      FileUtils.mkdir_p(string)
+      FileUtils.mkdir(string)
     end
 
     def self.open(string, &block)
